\ Clean Songlist - for the Digital Piano -- include 3iPiano.fs  

\ TODO: ADD REST NOTATIONS
\ TODO: ADD variable tones (half and sixteenth notes)
\ TODO: FIX - leave doesn't work
\ Design: this plays by reading notes or directly e.g. 1 .. 2 .. 3 ..

: include-song-files
  s" 3iForth.fs"        included  500 ms
  s" 3iSound-4x4.fs"    included  500 ms  \ includes 3iMusic-scales
  s" 3iShift.fs"        included  500 ms
  s" 3iPianoLights.fs"  included  500 ms
;

include-song-files  \ includes 3iSongs, 3iMusic-menu

\ MUSICAL NOTATION FOR SONGLIST.fs
\ Design: SONG GRAMMAR consists of (1) 8 diatonic notes 1..8,  (2) 6 chromatic notes 9-14, (3) octave down/up, (4) musical rest short/long
\ Design: the grammar allows writing music in easy-to-read 1-8 notation, #/% for sharp/flat, <</>> for octave down/up, and //,///,//3,//// for short/long musical rest
\ Design: scaleroot sets default index to middle c  (c_ / 36 note or 262 Hz tone), span c_ to c'
\ Design: scaleroot gets adjusted to move virtually between octaves, avoiding physically moving these
variable scaleroot 
: scaleroot!      ( note-name -- ) scaleroot !    ;
: scaleroot@      ( -- note-name ) scaleroot @    ;
: reset-scaleroot ( -- )           c_  scaleroot! ;
reset-scaleroot

\ Design: 1 2 ... 8 notation for white keys indexed to middle c_
\ Design: Sharps and Flats added through decorating 1..8
\ Design: cmaj-map maps 1...8 (white keys) and 9..14 (black keys) to the sequential piano note index
9  constant 1#
9  constant 2%
10 constant 2#
10 constant 3%
11 constant 4#
11 constant 5%
12 constant 5#
12 constant 6%
13 constant 6#
13 constant 7%
14 constant 8#

\ CODE REVIEW: use >mem
\ map: diatonic to chromatic, i.e. allows use of 1..8
create cmaj-map 14 bytes allot
white-registers black-registers 13 14 cmaj-map >mem drop
: [...]  ( playlist# -- 0index# )     1 - cmaj-map + c@ ;  \ transforms playlist notation to sequential notes 0-indexed

\ CODE REVIEW: what is this for?  reading notes from device?  but then change so not have first black key on 10, but rather 9, and do away with this...
create note-to-0index 14 bytes allot
white-registers -1 black-registers 14 note-to-0index >mem drop 
: translatenote ( recordnote -- 0indexnote )  \ takes songlist grammar note [diatonic] (1..14) and converts to piano sequence (0..13) [chromatic]
    1 - bytes note-to-0index + c@ ;
\ : test 15 1 do i dup . translatenote dup . _soundlight 200 ms cr loop ;

\ Design: this is the inverse map: chromatic to diatonic.  Used for?
create chr2cmaj-map  14 bytes allot
1 9 2 10 3 4 11 5 12 6 13 7 8 14  14 chr2cmaj-map >mem drop
: chr2dia ( 0indexnote -- grammarnote )  \ takes 0index chromatic note and converts to 1-index cmaj note
    bytes chr2cmaj-map + c@ ;
\ : test 13 0 do i dup . chr2dia . cr loop ;

\  Design: on 32-key F__ to C'' keyboard seq# is -7 to 24  (piano-led board)
: [,,,]  ( 1-8# or 9-14# -- 0index# ) 1 - scale + c@ c_ - ;   \ provides relative index +/- of note referenced to c_ middle c for transposition

\ Design: non-note grammar are above the c.96 notes in the piano scale (extended manually to custom frequencies)
\ Design: virtual (software only) octave adjustments.  See ><< and >>> in 3iSound-4x4.fs for physical/hardware octave adjustments in buttons
$FA constant <<  \ octave down
$FB constant >>  \ octave up
: is-octave? ( g -- g 0/1 ) dup << >> between? ;
: [<<] ( -- ) scaleroot@ 12 - scaleroot! ;   \ lower octave
: [>>] ( -- ) scaleroot@ 12 + scaleroot! ;   \ raise octave
: adjust-octave ( g:<<,>> --  ) >> - if [<<] else [>>] then ;

\ Design: musical rests
$FC constant //    \ short rest R1
$FD constant ///   \       rest R2
$FE constant 3///  \       rest R3
$FF constant 4///  \ long  rest R4
: >// 200 ms ;
create rests ' R4 , ' R3 , ' R2 , ' R1 ,
: is-rest? ( g -- g 0/1 ) dup // 4/// between? ;
: musical-rest ( g --  ) $FF swap - cells rests + @ execute ;

\ Design: storage for SONG GRAMMAR g for playback, transpositions etc.
create songmem 512 bytes allot
variable songptr
variable songlen
: songlen@ songlen @ ;
: reset-songlen   ( -- )   0 songlen ! ;
: fresh-song      ( -- )   songmem songptr !  reset-songlen ;
: songlen++       ( -- )   songlen@ 1 + songlen ! ;
: songc!+         ( g -- ) songptr @   c!+  songptr ! songlen++ ;
: song-mem2list   ( n -- list n )  \ memory-array to list-stack
    >r songmem r@ 0 do c@+ swap loop drop r> ;
: showsongmem     ( -- ) songlen@ songmem over 0 do c@+ . loop drop ;  \ dump-memory
: showsonglist    ( n -- ) dup 0 do dup i - roll . loop drop ;  \ dump-list
fresh-song

\ OBSERVATION:  5flats  lowered 1 semitone gives a nice cmaj-white keys scale!

: pausedur 500 ms ;

: _key              ( -- 0/1 )    key esc-key = if leave then ;  \ BUG!!! esc-key needs to trigger leaving outerloop
: _sound            ( note# -- )  note ;            \ transform to note# (3iSound-4x4.fs) and plays note
: [light]           c_ - 8 + ;    \ calibrated to middle C
: _light            ( note# -- )  [light] kk ;     \ transform to light# (3iPianoLights.fs) and turns on light
: _soundlight       ( note# -- )  dup _light _sound ;
: _soundlightkey    ( note# -- )  dup . _soundlight _key ; \ advance on keypress, ESC to stop
: _soundlightbutton ( note# -- )  
    begin dup _soundlight poll-note-data translatenote scaleroot@ + 2dup = 
        if drop pausedur exitloop 
        else drop clear-32 pausedur pausedur continueloop 
        then 
    until drop ;
: _lightkey         ( 0index# -- )  _light _key ;
: _soundkey         ( 0index# -- )  _sound _key ;

\ Design mode: playmode is a CONFIGURATION PARAMETER - Hw menu
variable playmode
0 constant  soundonly
1 constant  lightonly
2 constant  soundlight
3 constant  soundlightkey
4 constant  lightkey
5 constant  soundkey
6 constant  soundlightbutton
: playmode!  ( choice -- ) playmode ! ;
: soundonly-mode  soundonly playmode! ;
: playmode@  ( -- choice ) playmode @ ;
: soundlight-mode soundlight playmode! ;
: soundlightkey-mode soundlightkey playmode! ;
: soundlightbutton-mode soundlightbutton playmode! ;
: soundkey-mode soundkey playmode! ;
: lightkey-mode lightkey playmode! ;
soundlight-mode  \ default at startup

create playmode-map   ' _sound , ' _light , ' _soundlight , ' _soundlightkey , ' _lightkey , ' _soundkey , ' _soundlightbutton ,
  \ Design: .. doesn't always sound a note, if << or >> then it adjusts the octave and stops as the next note needs to be rolled in from the stack
: do-playmode ( note# -- ) playmode-map playmode@ cells + @ execute ;

\ CODE REVIEW: Get to a musical data structure -- 3 arrays
\ play directly from stack, left to right


\ ================= ALGORITHM -- EXTRACT / DISPLAY KEY SIGNATURE FROM SONG
\ \ Design: extract uniques
create songuniq 45 bytes allot \ enough to surround the 32-key pianolights board
variable uniqptr
variable uniqlen
: reset-uniqptr  songuniq uniqptr !   0 uniqlen ! ;
: uniqptr@   uniqptr @ ; 
: uniqptr++  uniqptr@ 1 + uniqptr ! ;
: uniqlen@  uniqlen @ ;
: uniqlen++  uniqlen @ 1 + uniqlen ! ;
: add-uniq ( x -- ) uniqptr@ ! uniqptr++ uniqlen++  ;  \ ignore non-note grammars
: [show-uniq] ( -- ) uniqlen@ 0 > if songuniq uniqlen@ 0 do c@+ . loop drop else ." empty " then ;

: end?  ( uptr -- 1/0 ) dup uniqptr@ = ;
: uniq? ( x uptr -- x uptr' 1/0 ) over swap c@+ rot = ;  \ CODE REVIEW: this is actually 0 for uniq, 1 for not uniq --- misnamed
: search-uniq ( x uptr -- 0/1 )     
    begin uniq?
        if  \ not-unique, i.e. matching entry found
            drop exitloop
        else \ uniq, i.e. entry not found, continue search unless at end of list
            end? 
                if \ at end of list
                    swap add-uniq exitloop 
                else continueloop 
                then 
        then 
    until
    drop
;
: extract-uniqs ( songlen -- uniqlen ) \ 
     reset-uniqptr songmem swap 0 
     do \ loop over song grammar elements
        c@+ dup << < 
            if songuniq end?   \ found note
                if drop add-uniq \ reached end of list, so add
                else search-uniq
                then
            else drop \ ignore non-notes
            then 
    loop
    drop 
    uniqlen@
;


: play-uniqs ( uniqlen -- uniqlen )
  songuniq over 0 do c@+ note R2 loop drop ;
\ CODE REVIEW: need ORDER algorithm
: [uniqlights] ( uniqlen -- list uniqlen ) 
    >r songuniq r@ 0 do c@+ [light] swap loop drop r> ;
: uniqlights ( uniqlen -- ) [uniqlights] lightn ;
: songlights ( songlen -- songlen ) dup extract-uniqs uniqlights ;

\ TODO -- translate to light IDs and then light them!
\ ================= END UNDER CONSTRUCTION


\ Design: Takes song grammar from stack and writes notes to memory FIXED WRT TO A SCALE and INTERPRETING virtual octaves, adding rests grammar.  
\             Example: dream [writesong] play
: [writenote]     ( g -- )
        is-octave? 
        if   adjust-octave     \ CODE REVIEW: add light for adjusting octave
        else is-rest?
              if  ( pass through )
              else [,,,] scaleroot@ + then
              songc!+  \ CODE REVIEW: count use of the array so know the length
        then 
;
: [writesong]     ( <1..8# notes> n -- ) 
    reset-scaleroot fresh-song  dup 0 
    do 
        dup i - roll 
        [writenote]
    loop drop ;

: list-uniqs ( <list> songlen -- uniqlen ) [writesong] extract-uniqs [show-uniq] uniqlen@ dup . ;

\   Design: songlist puts the entire song notation onto the stack and then reads from stack left to right  CODE REVIEW: store into memory, read from memory
\   Design: core musical dependency is 3iSound-4x4.fs which has the notational transformation required
\   Design: audio only
: .. ( grammar -- )  \ notation is 3iSonglist grammar: 1..8 2% 2# >> << // /// //3 ////  CX!!! in pace mode, extra delay because // pauses are read!
    is-rest?  if musical-rest
             else do-playmode
             then ;
: flashlights  ( -- ) on-32 300 ms clear-32 200 ms ;

\ Design: transposition is done from note index space to avoid having to translate back and forth to songlist grammer (user notation space) which would change length of songmem as octave movements would be different
\ Design: only transpose notes (not rests).  Transposition is 1-1 and cells are overwritten with results, songlen unchanged
: c<!+ ( x ptr -- ptr ) 1 - c!+ ;  \ overwrite previous character with x
: [ntranspose] ( semi# -- ) 
    set-semi# songlen@ songmem over 0 
    do 
        c@+   is-rest? if ( ignore ) else get-semi# + then swap c<!+ 
    loop 2drop ;

\ Design: these actions work on song notes and rests stored in memory
: replay      ( -- ) cr ." Playing..." songlen@ songmem over 0 do c@+ .. loop 2drop ." OK" ;  \ repeats song in memory
: [showuniqs] ( -- ) songlen@ dup extract-uniqs uniqlights drop ;
: ntranspose  ( #steps -- ) [ntranspose] [showuniqs] ;
: transpose-up     ( -- ) 1 ntranspose ; \ single-step transpose
: transpose-down   ( -- ) -1 ntranspose ; \ single-step transpose
: [showsong] ( -- ) songlen@ songmem .mem drop ;


\ TODO: to get back to user space, rebaseline (song-delta) against c_ (middle c) and use the chr2dia inverse map !!


\ TODO: calculate new 1..8 notation with the revised scale...
\ TODO: algorithm to choose transposition that gives optimal fingering, e.g. c major 
\ TODO: illuminate to show all the (uniq) notes of a scale... to see if one likes the fingering -- DONE
\ TODO: store songs in EEPROM (non-volatile) for playing/replaying rapidly - may lead to timing issues
\ TODO: scan program and highlight all keys to expose the underlying scale based on keys requested
\ : transpose-test ( g -- )  \ shows 7 steps of transformation between notations.   5flats  1 6 5 7 -->  6 11 3 5
\    cr dup .  [,,,] dup . scaleroot@ + dup . 1 + dup . scaleroot@ - dup . 12 mod dup . chr2dia dup . drop ;

\ Design: Song user interface to entry is 1..8,//,///,//3,////,>>,<< grammer loaded onto stack, written to memory, play, transpose, analysis is from memory
\ Design: songlen@ holds length of song (forming object with songmem)
\ Design: these actions work on song in list
: play        ( <songlist> -- )    cr ." playing (sound only)..."      soundonly-mode        [writesong] replay ;
: showsong    ( <songlist> -- )    [writesong] [showsong] ;
: [watch]     ( <songlist> -- )    [writesong] replay ;
: watch       ( <songlist> -- )    cr ." watching (sound & lights)..." soundlight-mode       [watch] ;
\ TODO - 2021-09-04---Sat---10:41 when pacing, show the key 1..8 (user space), not the note
: pace        ( <songlist> -- )    cr ." setting pace of click-through playback..."  soundlightkey-mode    [writesong] replay ; \ learns the pace to play it at, keyboard control

\ Design: ensuring this doesn't clobber settings
: victory-jingle  dur_ms @ >r playmode@ >r 80 setbeat  1 3 5 8 // 2 4 6 >> 2 <<   11 play r> playmode! r> setbeat ;
: well-done!   flashlights flashlights  victory-jingle ;

: learn       ( <songlist> -- )    cr ." learn song..."  soundlightbutton-mode [writesong] replay  well-done! ; 
: listenstep  ( <songlist> -- )    soundkey-mode         [writesong] replay ;
: lightstep   ( <songlist> -- )    lightkey-mode         [writesong] replay ;
: showuniqs   ( <songlist> -- )                          [writesong] [showuniqs] ; 

: dummy      ( -- ) cr ." no command" ;
: exitchoice ( -- ) exitloop ;
create songsdo
' replay          ,  ' [showuniqs]         ,  ' transpose-up          ,  ' transpose-down       ,    \  
' soundlight-mode ,  '  soundlightkey-mode ,  ' soundlightbutton-mode ,  ' soundonly-mode ,   \  watch, pace, learn, play
' updown-slide    ,  ' flashlights         ,  ' ring-ring             ,  ' rtw-ccw ,
' dummy           ,  ' dummy               ,  ' dummy                 ,  ' exitchoice ,
: songmenu-actions
  cr ."  ACTIONS (13)"
  cr ."  ------------"
  cr ."  PLAY!          uniqs         transpose-up    transpose-down"
  cr ."  watch-mode     pace-mode     learn-mode      sound-only-mode"
  cr ."  updown-slide   flashlights   ring-ring       lights rtw-ccw "
  cr ."  xx             xx            xx              EXIT..."
;




\ interesting: starting on E makes a lot of complicated songs easy!  dream -1   stairway 4
\ TODO: Add a "next note" button in case the learn mode gets stuck...

\   Design: audio + lights
\  : watch ( n -- ) dup 0 do dup i - roll dup #_ .. loop drop clear-32 ;   \ see 3iPianoLights.fs for kk
\  : watch ( n -- ) dup 0 do dup i - roll #__ 500 ms light| 100 ms loop drop light| ;   \ see 3iPort-4x4.fs for light_

\   Design: learn each note and use PC to prompt next note --- LATER: read key to check it's the right one being pressed
\ : learn ( n -- ) dup 0 do dup i - roll dup #_ .. key drop loop drop clear-32 ;
\ : learn ( n -- ) reset-scaleroot dup 0 do dup i - roll .. key drop loop drop clear-32 ;

\ TO DO: add ESC to play rest.  Add back key to go back.  Add play a phrase key.
\ TO DO: how to flash?


\ see SongList for map to musical notes and multiple keys

\  CODE REVIEW ---- 2021-10-10---Sun---06:42  WARNING!!  IN PROGRESS - STRIPPING OUT CHORDS CODE AND REPLACING WITH ARPEGGIOS
\    need to fix  silence-2voices and twinkle-chords and get everything working
\    need to add natural reversal...

\ CHORDS & ARPEGGIOS
\ Main chord types (maj, min, dim, aug): https://www.schoolofrock.com/resources/keyboard/piano-chords-for-beginners
\ Triads: https://en.wikipedia.org/wiki/Triad_(music)
\ Sevenths: https://www.musictheory.net/lessons/45
\ Ninths: https://blog.discmakers.com/2020/12/ninth-chords-add-ambiance/
\ Elevenths: https://blog.discmakers.com/2021/03/eleventh-chords/
\ Arpeggio: https://en.wikipedia.org/wiki/Arpeggio

\ Design: chord shape (shp) is a CONFIGURATION PARAMETER - select in hardware
variable shp \ chord shape
: shp! ( shp-id -- ) shp c! ;
: shp@ ( -- shp-id ) shp c@ ;
\ Design: root note will be user selected from the keyboard.  Needed to light and transpose the chord.
variable rt  \ root note
: rt! ( key-id -- ) rt c! ;
: rt@ ( -- key-id ) rt c@ ;

\ CODE REVIEW: add 9th and 11th and 13th chords (7 notes)

\ Design: chords are just arpeggios on a particular scale.  This simplifies the coding.  Furthermore, it is the incremental number of semitones between notes that is the key

\ major chord shape: root + 4 + 3
\ Design: this works for notes and lights.  Need to bypass the play, and go straight to reading from songmem

  
\ TODO: menu to choose chord shape/type  (e.g. x shp!)
\ TODO - design fast sweep of chords, and chord-mode option in Songlist.fs
\ CODE REVIEW: should be [,,] instead of translatenote (fix it in the .avr listing the keyprpesses)
\ extract-note returns 1--8 white keys and 10-14 black keys
\ : fix-defect ( extract-note ) dup 9 > if 1 - then ;  \ needed to fix previous key mapping 1-8, 10-14

\ CODE-REVIEW: fix to trigger on DOWN PRESS not UP PRESS

\ TODO: this provides a nice record notes functionality!  Now add edit notes

\ CHORDATED ARPEGGIO
\ Beautiful Example by mysid: https://upload.wikimedia.org/wikipedia/commons/8/80/Arpeggio.ogg
\ Question: https://en.wikipedia.org/wiki/User_talk:Mysid#Your_awesome_arpeggio_piece_(flute_&_harp!)
\ Design: arpeggiator v1 looped over the notes in a CHORDS, e.g. silence 1 3 2 3 (1-index)
\ Design: arpeggiator v2 loops over semitone increments, so is more versatile in what can be played, e.g. 0 4 3 5 is a major chord
\ Design: how to get reversed movement
create arpeg 65 bytes allot \ first cell is nlen, max length is 4 bars, each bar is 16 semiquavers, so 64 slots
: >narpeg ( <arpeg> n -- n ) dup arpeg c!+ >mem drop ;
\ : arpeg-chord  ( root -- chrd-root ) dup write-chord drop arpeg c@+ 0 do c@+ 1 - bytes chordmem + c@ .. loop drop ;
\ : ;; ( root -- ) dup light-chord arpeg-chord drop ;
: arpeg-1232  1 2 3 2  4 >narpeg ; \ standard chord arpeggio
: arpeg-1323  1 3 2 3  4 >narpeg ; \ picking pattern for sound of silence

\ Chords as Arpeggios
: major             0 4 3      3 >narpeg ;
: arpeg-maj5        0 7        2 >narpeg ;
: arpeg-major       0 4 3 5    4 >narpeg ;
: arpeg-maj7        0 4 3 4    4 >narpeg ;
: arpeg-domin7      0 4 3 3    4 >narpeg ;
: arpeg-augmen      0 4 4 4    4 >narpeg ;
: minor             0 3 4      3 >narpeg ;
: arpeg-minor       0 3 4 5    4 >narpeg ;
: arpeg-min7        0 3 4 4    4 >narpeg ;
: arpeg-halfdim7    0 3 3 4    4 >narpeg ;
: arpeg-dimin7      0 3 3 3    4 >narpeg ;
: arpeg-dimin       0 3 3 6    4 >narpeg ;
: arpeg-sus4        0 5 2 5    4 >narpeg ;
: arpeg-sus2        0 2 5 5    4 >narpeg ;

: arpeg-majupdown   0 4 3 5 $F5 $F3 $F4    7 >narpeg ;
: arpeg-maj1232     0 4 3 $F3 $F4          5 >narpeg ;
: arpeg-maj1323     0 7 $F3 3              4 >narpeg ;    \ sound of silence
: arpeg-min1323     0 7 $F4 4              4 >narpeg ;    \ sound of silence
: arpeg-majrev30   12 $F8 $F4              3 >narpeg ;    \ falling oct third root
: arpeg-majrev50   12 $F5 $F7              3 >narpeg ;    \ falling oct fifth root  mysterious fairy sound - mysid demo
: arpeg-majrev530  12 $F5 $F3 $F4          4 >narpeg ;    \ falling oct fifth third root
: arpeg-majrev54   12 $F5 $F2              3 >narpeg ;    \ falling oct fifth fourth
: arpeg-majrev53   12 $F5 $F3              3 >narpeg ;    \ falling oct fifth third
: arpeg-minrev53   12 $F5 $F4              3 >narpeg ;    \ falling oct fifth min3rd
: arpeg-o3         $FC 12 12               3 >narpeg ; 
: arpeg-o4         $FC 12 12 12            4 >narpeg ;
: arpeg-fallo4     24  $FC $FC $FC         4 >narpeg ;
: arpeg-fallo3     24  $FC $FC             3 >narpeg ; 
                                           
: arpeg-beat2       0   0                  2 >narpeg ;
: arpeg-beat2b    $FC 12                   2 >narpeg ;
: arpeg-beat2a     12 $FC                  2 >narpeg ;
: arpeg-beat3       0   0 0                3 >narpeg ;
: arpeg-beat3b    $FC 12 0                 3 >narpeg ;
: arpeg-beat4b    $FC 12 0 0               4 >narpeg ;

create arpeg-exec 
' major           , ' arpeg-major      , ' arpeg-maj7      , ' arpeg-domin7   , 
' arpeg-augmen    , ' minor            , ' arpeg-minor     , ' arpeg-min7     , 
' arpeg-halfdim7  , ' arpeg-dimin7     , ' arpeg-dimin     , ' arpeg-sus4     , 
' arpeg-sus2      , ' dummy            , ' dummy           , ' exitchoice     , 
                                       
' arpeg-majupdown , ' arpeg-maj1232    , ' arpeg-maj1323   , ' arpeg-majrev30 , 
' arpeg-majrev50  , ' arpeg-majrev530  , ' arpeg-majrev54  , ' arpeg-majrev53 ,
' arpeg-minrev53  , ' arpeg-o3         , ' arpeg-o4        , ' arpeg-fallo4   ,
' arpeg-fallo3    , ' dummy            , ' dummy           , ' exitchoice     ,
                                                           
' arpeg-beat2     , ' arpeg-beat2a     ,  ' arpeg-beat2b   , ' arpeg-beat3    ,
' arpeg-beat3b    , ' arpeg-beat4b     ,  ' dummy          , ' dummy          ,
' dummy           , ' dummy            ,  ' dummy          , ' dummy          ,
' dummy           , ' dummy            ,  ' dummy          , ' exitchoice     ,
                                       
' dummy           , ' dummy            ,  ' dummy          , ' dummy          ,
' dummy           , ' dummy            ,  ' dummy          , ' dummy          ,
' dummy           , ' dummy            ,  ' dummy          , ' dummy          ,
' dummy           , ' dummy            ,  ' dummy          , ' exitchoice     ,

: arpeglist 
cr ." LIST A: (all chords are root+third+fifth+octave unless flagged [3] i.e. without octave)"
cr ."    1-4:     major[3]       major           maj7          domin7   " 
cr ."    5-8:     augmen         minor[3]        minor         min7     " 
cr ."   9-12:     halfdim7       dimin7          dimin         sus4     " 
cr ."  13-16:     sus2           xx              xx            EXIT...  " 
cr ." LIST B: (arpeggios)"
cr ."    1-4:     majupdown      maj1232         maj1323       majrev30  " 
cr ."    5-8:     majrev50       majrev530       majrev54      majrev53 "
cr ."   9-12:     minrev53       o3              o4            fallo4   "
cr ."  13-16:     fallo3         xx              xx            EXIT...  "
cr ." LIST C (beats):"
cr ."    1-4:     beat2          beat2a          beat2b        beat3    "
cr ."    5-8:     beat3b         beat4b          xx            xx       "
cr ."   9-12:     xx             xx              xx            xx       "
cr ."  13-16:     xx             xx              xx            EXIT ... "
cr ." LIST D:"                                                 
cr ."    1-4:     xx             xx              xx            xx       "
cr ."    5-8:     xx             xx              xx            xx       "
cr ."   9-12:     xx             xx              xx            xx       "
cr ."  13-16:     xx             xx              xx            EXIT ... "
cr
;


\ Design: independent tempo control for arpeggios, good is 50 or 120
\ Design: duration of 0 for continuous tone
variable arpeg-tempo
: atempo!  arpeg-tempo ! ;
: atempo@  arpeg-tempo @ ;
: atempo ( bpm -- ) atempo@ [bpm] setbeat ;
700 atempo!
arpeg-majrev53
\ Design: if $Fh then negative step h.  Still allows multiple octave sweeps
: arpeg-semistep   ( root -- ) arpeg c@+ 0 do c@+ rot swap dup $F0 > if $0F and - else + then dup .. swap loop 2drop ;
: '' ( rootnote -- ) dup is-rest? if musical-rest drop else [,,] arpeg-semistep 2drop then ; \ shorthand for playing chord
: ;; ( root -- ) beat >r  atempo arpeg-semistep r> setbeat ;
: arpeggiator ( -- )
   ." Entering Arpeggio mode (any key to quit)" cr
   1 silent  \ mute keys
   begin
    fetch-note-data 0> if extract-note dup rt! [,,] dup . ;; then key?
   until key drop clear-32 
   0 silent ;

\ Design: ANSI Forth defines: http://lars.nocrew.org/forth2012/facility/KEYq.html
\ : testkey begin begin key? until ." 1" key drop 500 ms key? until ." 2" key drop ;

\ see arpeg-demo-mysid for cool results!

\ CODE-REVIEW: create loop for beats track... merging arpeggios and beats, introducing rests.

\ Demos
\ Design: these arpeggios play over the notes of a chord triggered by user pressing root key
\ Design: [light] conversion of note to light space is in Songlist.fs
\ Design: major chords are 0 4 7 semitones
\ Use c_ ;; to run arpegio and light triad
\ Use arpeggiator to run the above triggered by button presses
: arpeg-demo 1 3 3 2 3 3 1 3 2 3 3 2 1 3 2 2 1  17 >narpeg arpeggiator ;

\ ex. c-maj = C E G ;  \ ex. c-maj7 = C E G B ;

\ TODO - add arpeggio structure eg //1353//
\ TODO - add note names for chords... instead of sequences...

\ CODE REVIEW - MAY NEED TO FIX NOW THAT THIS IS CHANGED

\ To transpose the chord, move the shape
: chf ( -- ) rt@ 1 + dup rt! arpeg-semistep ;
: chb ( -- ) rt@ 1 - dup rt! arpeg-semistep ;


\ Demos
\ : demo-musiclights
\   500 >r 
\   clear-32 r@ ms
\   8 kk     r@ ms
\   5 kk     r@ ms
\   1 kk     r@ ms
\   3 kk     r@ ms
\   
\   8 kk     r@ ms
\   49 light-chord r@ ms
\   5 kk     r@ ms
\   17 light-chord r@ ms
\   1 kk     r@ ms
\   13 light-chord r@ ms
\   3 kk     r@ ms
\   15 light-chord    r@ ms
\   
\   3 kk     r@ ms
\   5 kk     r@ ms
\   7 kk     r@ ms
\   8 kk     r@ ms
\   r> drop 
\ ;


: help-songlist
cr 
cr ." HELP-SONGLIST"
cr ." -------------"
cr ." by Assad Ebrahim - http://www.mathscitech.org/articles"
cr 
cr ." include 3iSonglist.fs  [3iForth.fs 3iCore.fs 3iSound-4x4.fs 3iMusic-scales.fs 3iShift.fs 3iPianoLights.fs]"
cr ." include 3iArpeggiator.fs    - for examples playing over scale"
cr ." init: 1 open-com  init-songs"
cr ." playlist"
cr ."      play | watch | pace | learn | showuniqs | transpose | ntranspose "
cr 
cr ." Tools: "
cr ."    replay   e.g. after transpose or ntranspose"
cr ."    ><<   >>>  octave down/up"
cr ."    help-scales   set white keys to diatonic scale for ethnomusic"
cr ."    help-theory   show grammar / words for music theory"
cr ."    major minor sus4 d_ playchord  chordator"
cr ."    <seq#s> n >narpeg arpeggiator  (on chords)"
;

: help-theory
cr
cr ." Tools for Music Theory"
cr ." ----------------------"
cr ." Song notation (grammar): 1..8, #,% <<,>> //,///,//3,//// - notes, sharp/flat, octave down/up, rests short/long"
cr ." [writesong]  - converts to piano note grammar, "
cr ." extract-uniqs  show-uniq"
cr ." list-uniqs"

cr
;

help-songlist
help-theory

cr
cr ." INSTRUCTIONS:"
cr ."   1.   1 open-com"
cr ."   2.   init-songs"
cr 

\ ============ INITIALIZE

200 constant msg-delay
: init-songs ( -- )
  cr ." 1) init speaker at pin " 13 dup . ." ... " dup out set-speaker ." OK" msg-delay ms
  cr ." 2) set scale to cstd ... " 	cstd play-scale  ." OK" msg-delay ms    \ nice chord: b.. d. g. 
  cr ." 3) init shift out registers ... " init-shiftout flashlights ." OK" msg-delay ms
  cr ." 4) init shift in registers ..." init-shiftin ." OK" msg-delay ms
  cr 
  s" 3iSongs.fs" included 
  cr
  cr ." 5) loading songs playlist..."   ." OK" msg-delay ms
  cr ." 6) setting sound & light mode on ..." soundlight-mode ." OK" msg-delay ms
  cr ." 7) setting piano serial on ..." 1 buttonsrecord ." OK" msg-delay ms  well-done!
  cr ." 8) setting defaults"
  cr ."      arpeggio/chord mode to major and arpeggio tempo to 50..." major 50 atempo! ." OK" msg-delay ms
  cr ."      beat tempo to 120 bpm..." 120 bpm  ." OK" msg-delay ms
  s" 3iMusic-menu.fs" included msg-delay ms
  cr ." 9) type: music-menu for help" msg-delay ms
  cr 
;


\ include 3iSongs.fs
\ include 3iMusic-menu.fs

\ V3 - 2021-05-16---Sun---17:14 - perfected the multi-octave notation with << and >> mapping to [<<] and [>>]
\ V4 - 2021-08-11---Wed---06:28 - MAJOR - perfected integration with 32-key piano lights board and different key signatures (ethnoscales), fixed songlist
\ V5 - 2021-08-20---Fri---07:04 - MAJOR - FEATURE (3iSonglist): Added true teach-and-learn capability to the piano! using poll-note-data (3iArpeggiator) to fetch data
\ V6 - 2021-08-21---Sat---08:30 - MAJOR - PARTIAL FEATURE: 
\ IN PROGRESS: partial transpose capability - runs into trouble juggling handling octave.
\    To Reproduce bug from I have a dream
\    5flats
\    6 7 8 16 2 1 15 5     8 play
\    6 7 8 16 2 1 15 5     8 0 ntranspose-song
\    cstd
\    4 11 12 15 13 16 15 12 16 10   10 play
\ V7 2021-08-24---Tue---01:36 MAJOR REVISION: play/watch/pace/learn first [writesong] to mem then play from mem using notes
\  Song grammer is parsed to write once in the note (fixing scale and octave adjustments)
\  After than transpositions are easy in note format from memory
\ V8 2021-08-25---Wed---05:33 -- fixed learning mode and corrected playlist -- 55 songs
\ V9 2021-08-27---Fri---05:07 -- added extract-uniq algorithm, features: show key-signature (light up all keys needed to play), transpose by 1 semitone or n semitones, annotated all 55 songs with optimal transposition to stay entirely on white keys
\ V10 2021-09-14---Tue---22:16 -- added arpeggiator built on top of chordator!  in 8 lines of Forth code.
\ V11 2021-10-10---Sun---06:45 -- created much more flexible arpeggios capability to get sweeps and chords in ascending or descending order.  partially complete stripping out chords implementation
\ V12 -- 2021-10-13---Wed---05:09 - overhauling note, rest timing system


\ TODO - 2021-08-27---Fri---05:21 software is now good enough to design hardware user interface and test it out
\ TODO - 2021-09-02---Thu---23:13 add to >>> and ><< for cstd the movement of the black5's, with the check of 0 if 0
